#!/bin/sh

case "$1" in
'openldap1')
  # for OpenLDAP1
  TESTDIR=./test/openldap1
  LDAPD=/usr/local/libexec/slapd
  CONFIG=$TESTDIR/slapd.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  RUBY=/usr/local/bin/ruby
  LOGFILE=test.log
  SERVERLOG=serv.log
  LDAPDOPT="-d 2 -f $CONFIG -p $PORT"
  LDAPDCMD="$LDAPD $LDAPDOPT"
  ;;
'openldap2')
  # for OpenLDAP2
  TESTDIR=./test/openldap2
  LDAPD=/usr/local/libexec/slapd
  CONFIG=$TESTDIR/slapd.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  RUBY=/usr/local/bin/ruby
  LOGFILE=test.log
  SERVERLOG=serv.log
  LDAPURL="ldap://127.0.0.1:$PORT/"
  LDAPDOPT="-d 2 -f $CONFIG -h \"$LDAPURL\""
  LDAPDCMD="$LDAPD $LDAPDOPT"
  ;;
'openldap2-ssl')
  # -- experimental -- OpenLDAP2 with SSL,SASL
  TESTDIR=./test/openldap2
  LDAPD=/usr/local/libexec/slapd
  CONFIG=$TESTDIR/slapd-ssl.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  SSLPORT=6667
  RUBY=/usr/local/bin/ruby
  LOGFILE=test.log
  SERVERLOG=serv.log
  LDAPURL="ldaps://127.0.0.1:$SSLPORT/ ldap://127.0.0.1:$PORT/"
  LDAPDOPT="-d 2 -f $CONFIG -h \"$LDAPURL\""
  LDAPDCMD="$LDAPD $LDAPDOPT"
  EXT_SCRIPTS="test/bind-ssl.rb"
  OPENSSL=openssl
  CERTFILE=$TESTDIR/test.crt
  KEYFILE=$TESTDIR/test.key

  rm -f $CERTFILE $KEYFILE
  $OPENSSL req -new -x509 -out $CERTFILE -nodes -days 360
  mv privkey.pem $KEYFILE
  ;;
*)
  echo "$0 {openldap1|openldap2|openldap2-ssl(experimental)}"
  exit
esac


error(){
  echo $@ 1>&2
}

runruby(){
  echo "-- $RUBY $1 $PORT $SSLPORT --" >> $LOGFILE
  $RUBY $1 $PORT $SSLPORT 1>> $LOGFILE 2>> $LOGFILE
}


if [ `basename $PWD` = "test" ]; then
  error "run in the parent directory"
  exit
fi

# initialize DB
rm -rf $DBDIR
mkdir $DBDIR

# start slapd
rm -f $LOGFILE
rm -f $SERVERLOG

eval $LDAPDCMD 1>> $SERVERLOG 2>> $SERVERLOG &
echo $LDAPDCMD

while [ ! -f "$PIDFILE" ]; do
  echo "waiting for ldapd"
  sleep 1
done


SCRIPTS="test/bind.rb \
         test/add.rb \
         test/search.rb \
         test/search2.rb \
         test/search3.rb \
         test/modrdn.rb \
         test/search2.rb \
         test/delete.rb \
	 test/search2.rb"

if [ -f "$LOGFILE" ]; then
  rm $LOGFILE
fi

RESULT=ok
for f in $SCRIPTS
do
  printf "running $f .... "
  if runruby $f; then
    echo "succeed"
  else
    echo "fail"
    RESULT=error
  fi
done

for f in $EXT_SCRIPTS
do
  printf "running $f .... "
  if runruby $f; then
    echo "succeed"
  else
    echo "not supported"
  fi
done

# stop slapd
kill -TERM `cat $PIDFILE`
rm -rf $DBDIR
if [ "$CERTFILE" != "" ]; then
  rm -f $CERTFILE $KEYFILE
fi
